/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

char *
recode (char *a)
{
#ifdef HAVE_LIBRECODE
  size_t output_len = 0, output_alloc = 0;
  char *output = NULL;

  recode_buffer_to_buffer (request, a, strlen (a), &output, &output_len,
			   &output_alloc);

  if (!output && !(output = strdup (a)))
    fatal ("Error: memory.");

  return output;

#else

  return a;
#endif

}

size_t
irc_write (char *a, size_t b)
{
  size_t ret;

  pthread_mutex_lock (&mutex);

  if (debug)
    {
      write (STDERR_FILENO, a, b);

      if (a[b - 1] != '\n')
	write (STDERR_FILENO, "\n", 1);
    }

#ifdef HAVE_LIBSSL
  if (secure)
    {
      ret = SSL_write (ssl, a, b);

      if (a[b - 1] != '\n')
	SSL_write (ssl, "\n", 1);
    }
  else
    {
#endif
      ret = write (fd, a, b);

      if (a[b - 1] != '\n')
	write (fd, "\n", 1);
    }

  pthread_mutex_unlock (&mutex);

  return ret;
}

size_t
irc_read (char *a, size_t b)
{
  int ret;
  struct timeval tv;
  fd_set in;

  do
    {

      FD_ZERO (&in);
      FD_SET (fd, &in);

      tv.tv_sec = 0;
      tv.tv_usec = 10000;
    }
  while (!select (fd + 1, &in, NULL, NULL, &tv));

  pthread_mutex_lock (&mutex);

#ifdef HAVE_LIBSSL
  if (secure)
    ret = SSL_read (ssl, a, b);
  else
#endif
    ret = read (fd, a, b);

  pthread_mutex_unlock (&mutex);

  return ret;
}

/* EOF */
