/* indybot - Copyright (C) 2004 bakunin - <bakunin@autistici.org>
 *
 * This source code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Public License as published 
 * by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 *
 * This source code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * Please refer to the GNU Public License for more details.
 *
 * You should have received a copy of the GNU Public License along with
 * this source code; if not, write to:
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#else
# error Use configure; make; make install
#endif

#include <indybot.h>

void
check (struct channel *chan)
{
  struct file *fl;

  for (fl = chan->files; fl; fl = fl->next)
    {
      if (debug)
	fprintf (stderr, "Downloading file '%s' for '%s' channel...\n",
		 fl->file, chan->channel);

      parse (fl->file, chan, fl);
    }
}

int
parse_backslash_story (nxml_t * doc, nxml_data_t * cur, struct channel *chan,
		       struct file *fl)
{
  char *n_url, *n_title, *n_author;
  char *title, *author, *url;
  int len, id, i;
  char tmp[SIZE_BUFFER];

  title = url = author = NULL;

  for (cur = cur->children; cur; cur = cur->next)
    {
      if (cur->type == NXML_TYPE_ELEMENT)
	{
	  if (!strcmp (cur->value, "title") && !title)
	    nxml_get_string (cur, &title);

	  else if (!strcmp (cur->value, "url") && !url)
	    nxml_get_string (cur, &url);

	  else if (!strcmp (cur->value, "author") && !author)
	    nxml_get_string (cur, &author);
	}
    }

  if (!title || !url || !author)
    {
      if (title)
	free (title);

      if (url)
	free (url);

      if (author)
	free (author);

      return 0;
    }

  if (!(id = get_story_id (url)))
    return 0;

  for (i = 0; i < MAX_IDS && fl->last_ids[i]; i++)
    if (fl->last_ids[i] == id)
      {
	if (debug)
	  fprintf (stderr, "Check ID %d (position: %d)\n", id, i);

	return -1;
      }

  if (i < MAX_IDS)
    {
      fl->last_ids[i] = id;

      fprintf (stderr, "Cache empty: add ID %d (position: %d).\n", id, i);

      if (i == MAX_IDS - 1)
	return -1;

      return 0;
    }

  for (i = MAX_IDS - 1; i > 0; i--)
    fl->last_ids[i] = fl->last_ids[i - 1];

  fl->last_ids[i] = id;

#ifdef HAVE_LIBRECODE
  if (!doc->encoding || !strcasecmp (doc->encoding, charset))
    {
      char *tmp;

      if (doc->encoding)
	{
	  char buf[1024];

	  snprintf (buf, sizeof (buf), "%s..%s", doc->encoding, charset);
	  recode_scan_request (request, buf);
	}
      else
	{
	  char buf[1024];

	  snprintf (buf, sizeof (buf), "iso-8859-1..%s", charset);
	  recode_scan_request (request, buf);
	}

      n_url = recode (url);

      tmp = trim (title);
      n_title = recode (tmp);
      free (tmp);

      tmp = trim (author);
      n_author = recode (tmp);
      free (tmp);
    }
  else
    {
#endif
      n_url = strdup (url);
      n_title = trim (title);
      n_author = trim (author);
#ifdef HAVE_LIBRECODE
    }
#endif

  len =
    snprintf (tmp, SIZE_BUFFER, "PRIVMSG %s :[indymedia] %s - by %s - %s\n",
	      chan->channel, n_title, n_author, n_url);

  free (n_url);
  free (n_title);
  free (n_author);

  if (len > 0)
    irc_write (tmp, len);

  free (title);
  free (url);
  free (author);

  return 0;
}

void
parse (char *file, struct channel *chan, struct file *fl)
{

  nxml_t *doc;
  nxml_data_t *cur;

  if (nxml_new (&doc) != NXML_OK)
    {
      fprintf (stderr, "Error: nxml init.\n");
      return;
    }

  if (fl->proxy)
    nxml_set_proxy (doc, fl->proxy, fl->proxy_authentication);

  if (fl->authentication)
    nxml_set_authentication (doc, fl->authentication);

  if (fl->certfile && fl->certpassword)
    nxml_set_certificate (doc, fl->certfile, fl->certpassword, fl->cacert,
			  fl->verifypeer);

  nxml_set_user_agent (doc, PACKAGE " " VERSION);

  if (nxml_parse_url (doc, file) != NXML_OK)
    {
      fprintf (stderr, "Error: nxml parse url.\n");
      nxml_free (doc);
      return;
    }

  if (nxml_root_element (doc, &cur) != NXML_OK || !cur)
    {
      fprintf (stderr, "Error: no info in xml file.\n");
      nxml_free (doc);
      return;
    }

  if (!strcmp (cur->value, "backslash"))
    parse_backslash (doc, cur, chan, fl);
  else if (!strcmp (cur->value, "RDF") || !strcmp (cur->value, "rss"))
    parse_rss (file, chan, fl);
  else
    fprintf (stderr, "Error: parse xml file.\n");

  nxml_free (doc);
}

void
parse_backslash (nxml_t * doc, nxml_data_t * cur, struct channel *chan,
		 struct file *fl)
{
  if (debug)
    fprintf (stderr, "Check backslash xml structs...\n");

  for (cur = cur->children; cur; cur = cur->next)
    if (cur->type == NXML_TYPE_ELEMENT && !strcmp (cur->value, "story"))
      parse_backslash_story (doc, cur, chan, fl);

  if (debug)
    fprintf (stderr, "Parsing OK\n");
}

void
parse_rss (char *file, struct channel *chan, struct file *fl)
{
  mrss_t *mrss;
  mrss_item_t *item;
  mrss_options_t *options;

  if (debug)
    fprintf (stderr, "Check rss xml structs...\n");

  if (!
      (options =
       mrss_options_new (0, fl->proxy, fl->proxy_authentication, fl->certfile,
			 fl->certpassword, fl->cacert, fl->verifypeer,
			 fl->authentication, PACKAGE " " VERSION)) != MRSS_OK)
    {
      if (debug)
	fprintf (stderr, "Rss error!\n");
      return;
    }

  if (mrss_parse_url_with_options (file, &mrss, options) != MRSS_OK)
    {
      if (debug)
	fprintf (stderr, "Rss error!\n");

      mrss_options_free (options);
      return;
    }

  mrss_options_free (options);

  for (item = mrss->item; item; item = item->next)
    if (parse_rss_item (mrss, item, chan, fl) == -1)
      break;

  mrss_free (mrss);

  if (debug)
    fprintf (stderr, "Parsing OK\n");
}

int
parse_rss_item (mrss_t * mrss, mrss_item_t * item, struct channel *chan,
		struct file *fl)
{
  char *n_url, *n_title, *n_description = NULL;
  char tmp[SIZE_BUFFER];
  int len, id, i;

  if (!item->title || !item->link || (description && !item->description))
    return 0;

  if (!(id = get_rss_id (item)))
    return 0;

  for (i = 0; i < MAX_IDS && fl->last_ids[i]; i++)
    if (fl->last_ids[i] == id)
      {
	if (debug)
	  fprintf (stderr, "Check ID %d (position: %d)\n", id, i);

	return -1;
      }

  if (i < MAX_IDS)
    {
      fl->last_ids[i] = id;

      fprintf (stderr, "Cache empty: add ID %d (position: %d).\n", id, i);

      if (i == MAX_IDS - 1)
	return -1;

      return 0;
    }

  for (i = MAX_IDS - 1; i > 0; i--)
    fl->last_ids[i] = fl->last_ids[i - 1];

  fl->last_ids[i] = id;

#ifdef HAVE_LIBRECODE
  if (!mrss->encoding || !strcasecmp (mrss->encoding, charset))
    {
      char *tmp;

      if (mrss->encoding)
	{
	  char buf[1024];

	  snprintf (buf, sizeof (buf), "%s..%s", mrss->encoding, charset);
	  recode_scan_request (request, buf);
	}
      else
	{
	  char buf[1024];

	  snprintf (buf, sizeof (buf), "latin1..%s", charset);
	  recode_scan_request (request, buf);
	}

      n_url = recode (item->link);

      tmp = html_trim (item->title);
      n_title = recode (tmp);
      free (tmp);

      if (description)
	{
	  tmp = html_trim (item->description);
	  n_description = recode (tmp);
	  free (tmp);
	}
    }
  else
    {
#endif
      n_url = strdup (item->link);
      n_title = html_trim (item->title);

      if (description)
	n_description = html_trim (item->description);
#ifdef HAVE_LIBRECODE
    }
#endif

  if (description)
    len =
      snprintf (tmp, SIZE_BUFFER, "PRIVMSG %s :[%s] %s - %s - %s\n",
		chan->channel, mrss->title, n_title, n_description, n_url);
  else
    len =
      snprintf (tmp, SIZE_BUFFER, "PRIVMSG %s :[%s] %s - %s\n",
		chan->channel, mrss->title, n_title, n_url);

  free (n_url);
  free (n_title);

  if (description)
    free (n_description);

  if (len > 0)
    irc_write (tmp, len);

  return 0;
}

/* EOF */
